import { executeInPage, computeSelector } from "./utils";
import { AbstractEditor } from "./AbstractEditor";
export class AceEditor extends AbstractEditor {
    constructor(e) {
        super();
        // This function will be stringified and inserted in page context so we
        // can't instrument it.
        /* istanbul ignore next */
        this.getAce = (selec) => {
            const elem = document.querySelector(selec);
            const win_ace = window.ace;
            if (win_ace !== undefined) {
                return win_ace.edit(elem);
            }
            else if (Object.prototype.hasOwnProperty.call(elem, 'aceEditor')) {
                return elem.aceEditor;
            }
            else {
                throw new Error("Couldn't find AceEditor instance");
            }
        };
        this.elem = e;
        // Get the topmost ace element
        let parent = this.elem.parentElement;
        while (AceEditor.matches(parent)) {
            this.elem = parent;
            parent = parent.parentElement;
        }
    }
    static matches(e) {
        let parent = e;
        for (let i = 0; i < 3; ++i) {
            if (parent !== undefined && parent !== null) {
                if ((/ace_editor/gi).test(parent.className)) {
                    return true;
                }
                parent = parent.parentElement;
            }
        }
        return false;
    }
    getContent() {
        return executeInPage(`(${ /* istanbul ignore next */(getAce, selec) => {
            return getAce(selec).getValue();
        }})(${this.getAce}, ${JSON.stringify(computeSelector(this.elem))})`);
    }
    getCursor() {
        return executeInPage(`(${ /* istanbul ignore next */(getAce, selec) => {
            let position;
            const ace = getAce(selec);
            if (ace.getCursorPosition !== undefined) {
                position = ace.getCursorPosition();
            }
            else {
                position = ace.selection.cursor;
            }
            return [position.row + 1, position.column];
        }})(${this.getAce}, ${JSON.stringify(computeSelector(this.elem))})`);
    }
    getElement() {
        return this.elem;
    }
    getLanguage() {
        return executeInPage(`(${ /* istanbul ignore next */(getAce, selec) => {
            const ace = getAce(selec);
            return ace.session.$modeId.split("/").slice(-1)[0];
        }})(${this.getAce}, ${JSON.stringify(computeSelector(this.elem))})`);
    }
    setContent(text) {
        return executeInPage(`(${ /* istanbul ignore next */(getAce, selec, str) => {
            return getAce(selec).setValue(str, 1);
        }})(${this.getAce}, ${JSON.stringify(computeSelector(this.elem))}, ${JSON.stringify(text)})`);
    }
    setCursor(line, column) {
        return executeInPage(`(${ /* istanbul ignore next */(getAce, selec, l, c) => {
            const selection = getAce(selec).getSelection();
            return selection.moveCursorTo(l - 1, c, false);
        }})(${this.getAce}, ${JSON.stringify(computeSelector(this.elem))}, ${line}, ${column})`);
    }
}
