import { computeSelector, executeInPage } from "./utils";
import { AbstractEditor } from "./AbstractEditor";
export class MonacoEditor extends AbstractEditor {
    constructor(e) {
        super();
        this.elem = e;
        // Find the monaco element that holds the data
        let parent = this.elem.parentElement;
        while (!(this.elem.className.match(/monaco-editor/gi)
            && this.elem.getAttribute("data-uri").match("file://|inmemory://|gitlab:"))) {
            this.elem = parent;
            parent = parent.parentElement;
        }
    }
    static matches(e) {
        let parent = e;
        for (let i = 0; i < 4; ++i) {
            if (parent !== undefined && parent !== null) {
                if ((/monaco-editor/gi).test(parent.className)) {
                    return true;
                }
                parent = parent.parentElement;
            }
        }
        return false;
    }
    getContent() {
        return executeInPage(`(${ /* istanbul ignore next */(selec) => {
            const elem = document.querySelector(selec);
            const uri = elem.getAttribute("data-uri");
            const model = window.monaco.editor.getModel(uri);
            return model.getValue();
        }})(${JSON.stringify(computeSelector(this.elem))})`);
    }
    // It's impossible to get Monaco's cursor position:
    // https://github.com/Microsoft/monaco-editor/issues/258
    getCursor() {
        return Promise.resolve([1, 0]);
    }
    getElement() {
        return this.elem;
    }
    getLanguage() {
        return executeInPage(`(${ /* istanbul ignore next */(selec) => {
            const elem = document.querySelector(selec);
            const uri = elem.getAttribute("data-uri");
            const model = window.monaco.editor.getModel(uri);
            return model.getModeId();
        }})(${JSON.stringify(computeSelector(this.elem))})`);
    }
    setContent(text) {
        return executeInPage(`(${ /* istanbul ignore next */(selec, str) => {
            const elem = document.querySelector(selec);
            const uri = elem.getAttribute("data-uri");
            const model = window.monaco.editor.getModel(uri);
            return model.setValue(str);
        }})(${JSON.stringify(computeSelector(this.elem))}, ${JSON.stringify(text)})`);
    }
    // It's impossible to set Monaco's cursor position:
    // https://github.com/Microsoft/monaco-editor/issues/258
    setCursor(_line, _column) {
        return Promise.resolve();
    }
}
